#!/usr/bin/python

# forrestgui.py

import sys
from PySide import QtGui
from PySide import QtCore
from QtMobility.Location import *
from kml import Kml
import defaults
import time

class SettingsDialog(QtGui.QDialog):
	def Save(self):
		s = dict()
		s['interval'] = self.interval.text()
		s['trackdir'] = self.trackdir.text()

		self.main.settings.Save(s)
		self.close()

#	def Browse(self):
#		dialog = QtGui.QFileDialog()
#		dialog.setFileMode(QtGui.QFileDialog.Directory)
#		dialog.setOption(QtGui.QFileDialog.ShowDirsOnly, True)
#		dialog.setDirectory(self.trackdir.text())
#		if dialog.exec_():
#			dirs = dialog.selectedFiles()
#			print dirs
#			print dirs[0]
#		self.trackdir.setText(dirs[0])

	def __init__(self, parent=None, main=None):
		QtGui.QDialog.__init__(self, parent)
		self.setWindowTitle('Forrest TT Settings')
		self.main = main
		
		self.interval = QtGui.QLineEdit(self)
		self.interval.setText(repr(self.main.settings.Get('interval', 'int', 10)))
#		self.interval.setInputMask("99999;_")
		self.interval.setAlignment(QtCore.Qt.AlignRight)
				
		label = QtGui.QLabel("Fix Interval", self)
		label2 = QtGui.QLabel("seconds", self)
		button = QtGui.QPushButton("Save", self)
		button.clicked.connect(self.Save)

		hbox = QtGui.QHBoxLayout()
		hbox.addWidget(label)
		hbox.addWidget(self.interval)
		hbox.addWidget(label2)

		self.trackdir = QtGui.QLineEdit(self)
		self.trackdir.setText(self.main.settings.Get('trackdir', 'string', defaults.workdir))
		
		## The browse button is disabled, because the QFileDialog seems to crash
		## after a directory has been selected.
##		button2 = QtGui.QPushButton("Browse", self)
##		button2.clicked.connect(self.Browse)

		hbox2 = QtGui.QHBoxLayout()
		hbox2.addWidget(QtGui.QLabel("Tracks Dir"))
		hbox2.addWidget(self.trackdir)
##		hbox2.addWidget(button2)

		vbox = QtGui.QVBoxLayout(self)
		vbox.addLayout(hbox)
		vbox.addLayout(hbox2)		
		vbox.addWidget(button)
		self.setLayout(vbox)

		
class MainMenu(QtGui.QMenuBar):
	def __init__(self, parent=None):
		QtGui.QMenuBar.__init__(self, parent)

 		actionSettings = QtGui.QAction("Settings", self)
 		self.connect(actionSettings, QtCore.SIGNAL('triggered()'), parent.showSettings)
 		self.addAction(actionSettings)
		
class MainWindow(QtGui.QMainWindow):
	def showSettings(self):
		sd = SettingsDialog(self, self.main)
		sd.show()
		
	def GPSUpdated(self, geoPositionInfo):
		self.edSat.setText("Last GPS fix: "+geoPositionInfo.timestamp().toString())
		self.geoCoord = geoPositionInfo.coordinate()

		if (self.trackrunning == True):
			self.kml.addCoordinateToPath(self.geoCoord.latitude(), self.geoCoord.longitude(), self.geoCoord.altitude())
			
		self.edLat.setText("%f" % self.geoCoord.latitude())
		self.edLon.setText("%f" % self.geoCoord.longitude())
		self.edAlt.setText("%f" % self.geoCoord.altitude())

	def MsgBox(self, str):
		m = QtGui.QMessageBox()
		m.setText(str)
		m.exec_()

	def btnSaveClicked(self):
		if self.tracksaved == False and self.trackrunning == False:
			try:
				t = time.localtime()
				filename = "forrest-%04d%02d%02d%02d%02d%02d.kml" % (t.tm_year,t.tm_mon,t.tm_mday,t.tm_hour,t.tm_min,t.tm_sec)
				print "Writing out to: "+filename
				file = open(self.main.settings.Get("trackdir", "string", defaults.workdir)+"/"+filename, "w")
				file.write(self.kml.toString())
				file.close()
				self.edStat.setText("Track stopped (saved).")
				self.tracksaved = True
			except ValueError:
				self.MsgBox("Could not create track file.")
		else:
			m = QtGui.QMessageBox()
			m.setText("Track still running or already saved. Stop track before saving.")
			m.exec_()
			
		
	def StartGPS(self):
		try:
			self.edStat.setText("Starting GPS...")
			self.locationDataSource = QGeoPositionInfoSource.createDefaultSource(self)
			self.locationDataSource.setPreferredPositioningMethods(QGeoPositionInfoSource.SatellitePositioningMethods)
			self.locationDataSource.positionUpdated.connect(self.GPSUpdated)
			self.locationDataSource.setUpdateInterval(self.main.settings.Get("interval", "int", 10))
			self.locationDataSource.startUpdates()
			self.edStat.setText("GPS Started.")
		except:
			self.edStat.setText("Error in GPS init.")
			
	def btnStartStopClicked(self):
		if self.trackrunning == False:
			start = True
			if (self.tracksaved == False):
				# We didn't save a previous track, ask if we can 
				# overwrite the current track
				m = QtGui.QMessageBox()
				m.setText("Previous track wasn't saved. Continue?")
				m.setStandardButtons(QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
				r = m.exec_()
				if (r == QtGui.QMessageBox.No):
					start = False
			if (start == True):
				# Start a new track: create KML DOM and setup GUI
				try:
					self.kml = Kml()
					self.trackrunning = True
					self.tracksaved = False
					self.btnStartStop.setText("Stop")
					self.edStat.setText("Running track.")					
				except IOError:
					self.MsgBox("Could not create KML DOM. There should be a template.kml file in my working dir.")
		else:
			self.btnStartStop.setText("Start")
			self.trackrunning = False
			self.tracksaved = False
			self.edStat.setText("Track stopped (unsaved).")
		
	def btnMarkClicked(self):
		if self.trackrunning:
			if self.geoCoord:
				self.kml.addPlacemark(self.geoCoord.latitude(), self.geoCoord.longitude(), self.geoCoord.altitude())
				self.edStat.setText("Marked spot.")
			else:
				self.MsgBox("No GPS fix yet. Cannot mark spot.")
		else:
			self.MsgBox("No track running. Cannot mark spot.")
			

	def __init__(self, parent=None):
		QtGui.QMainWindow.__init__(self)
		self.main = parent
		self.setWindowTitle('Forrest Track Tracker')
		self.setMenuBar(MainMenu(self))
		
		# The Track Stuff
		self.trackrunning = False
		self.tracksaved = True			# True, because when we start, we don't have to check if it is saved
		self.geoCoord = None

		# The Side Buttons
		self.btnStartStop = QtGui.QPushButton("Start")
		self.btnSave = QtGui.QPushButton("Save Track")
		self.btnMark = QtGui.QPushButton("Mark Spot")
		self.btnStartStop.clicked.connect(self.btnStartStopClicked)
		self.btnSave.clicked.connect(self.btnSaveClicked)
		self.btnMark.clicked.connect(self.btnMarkClicked)

		# The Info Stuff
		lblStat = QtGui.QLabel("Status:")
		lblLat = QtGui.QLabel("Latitude:")
		lblLon = QtGui.QLabel("Longitude:")
		lblAlt = QtGui.QLabel("Altitude:")
		lblSat = QtGui.QLabel("GPS status:")
		self.edStat = QtGui.QLineEdit()
		self.edStat.setReadOnly(True)
		self.edStat.setText("Initialising...")
		self.edLat = QtGui.QLineEdit()
		self.edLat.setReadOnly(True)
		self.edLon = QtGui.QLineEdit()
		self.edLon.setReadOnly(True)
		self.edAlt = QtGui.QLineEdit()
		self.edAlt.setReadOnly(True)
		self.edSat = QtGui.QLineEdit()
		self.edSat.setReadOnly(True)

		# put in a generic Widget
		wi = QtGui.QWidget()
		self.setCentralWidget(wi)

		layComplete = QtGui.QGridLayout(wi)
		layComplete.addWidget(lblStat, 0, 0)
		layComplete.addWidget(self.edStat, 0, 1)		
		layComplete.addWidget(lblLat, 1, 0)
		layComplete.addWidget(self.edLat, 1, 1)
		layComplete.addWidget(lblLon, 2, 0)
		layComplete.addWidget(self.edLon, 2, 1)
		layComplete.addWidget(lblAlt, 3, 0)
		layComplete.addWidget(self.edAlt, 3, 1)
		layComplete.addWidget(lblSat, 4, 0)
		layComplete.addWidget(self.edSat, 4, 1)
		layComplete.addWidget(self.btnStartStop, 0, 2)
		layComplete.addWidget(self.btnSave, 1, 2)
		layComplete.addWidget(self.btnMark, 2, 2)
		
		# The GPS stuff
		self.StartGPS()

