#!/usr/bin/python

# kml.py
#
# KML class
# Copyright (C) 2010 Bart Friederichs

import sys
from xml.dom.minidom import parse
import time
import os

class Kml:
	# String Output Types
	KmlTypePlain = 0
	KmlTypePretty = 1

	def __init__(self):
		try:
			tpl = os.getcwd()+"/template.kml"
			self.dom = parse(tpl)
			elems = self.dom.getElementsByTagName("name")
			for elem in elems:
				text = self.dom.createTextNode("Forrest Track Tracker")
				elem.appendChild(text) 
		except IOError:
			print "Could not parse "+tpl
			raise

	# math.isnan is not available on our version of Python
	def isnan(self, f):
		return str(f)=='nan'
	
	def addPlacemark(self, lat, lon, alt):
		try:
			# we use UTC here
			t = time.gmtime()
			timestring = "%04d-%02d-%02dT%02d:%02d:%02dZ" % (t.tm_year,t.tm_mon,t.tm_mday,t.tm_hour,t.tm_min,t.tm_sec)		
			doc = self.dom.getElementsByTagName("Document")
			pm = self.dom.createElement("Placemark")
			ts = self.dom.createElement("Timestamp")
			wh = self.dom.createElement("when")
			nm = self.dom.createElement("name")
			pt = self.dom.createElement("Point")
			cd = self.dom.createElement("coordinates")
			
			doc[0].appendChild(pm)
			pm.appendChild(ts)
			ts.appendChild(wh)
			wh.appendChild(self.dom.createTextNode(timestring))
			pm.appendChild(pt)
			pm.appendChild(nm)
			nm.appendChild(self.dom.createTextNode("Marked at: "+timestring))
			pt.appendChild(cd)
			cd.appendChild(self.dom.createTextNode("%f,%f,%f" % (lon, lat, alt)))
		except:
			print "Exception in placing Placemark: %s" % sys.exc_info()[1]
	

	def addCoordinateToPath(self, lat, lon, alt):
		elems = self.dom.getElementsByTagName("coordinates")
		if (len(elems) == 1):
			if (self.isnan(lat)):
				lat = 0
			if (self.isnan(lon)):
				lon = 0
			if (self.isnan(alt)):
				alt = 0
		
			coords = "%f,%f,%f \n" % (lon, lat, alt)
			text = self.dom.createTextNode(coords)
			elems[0].appendChild(text)

	def __repr__(self):
		print "<KML object>"
		
	def toString(self, type=KmlTypePlain):
		return self.__str__(type)
	 
	def __str__(self, type=KmlTypePlain):
		r = None
		if (type == self.KmlTypePretty):
			r = self.dom.toprettyxml()
		else:
			r = self.dom.toxml()
		return r
		
			
		
